#include <engine/2d/gui.h>

/*
 * slider_init:
 *	Initializes the members of a Slider struct.
 */
void
slider_init(p_slider, p_screen_pos, p_texpage_pos, style, clut, tpage)
struct Slider *p_slider;
RECT *p_screen_pos, *p_texpage_pos;
enum SliderStyle style;
unsigned short clut, tpage;
{
	p_slider->x = p_screen_pos->x;
	p_slider->y = p_screen_pos->y;
	p_slider->w = p_screen_pos->w;
	p_slider->h = p_screen_pos->h;

	/* initialize "simple style" slider */
	SetLineF2(&p_slider->bar);
	setRGB0(&p_slider->bar, 0, 0, 0);
	setXY2(&p_slider->bar, p_screen_pos->x, p_screen_pos->y, p_screen_pos->x+p_screen_pos->w, p_screen_pos->y);

	SetSprt8(&p_slider->handle);
	setRGB0(&p_slider->handle, 128, 128, 128);
	setXY0(&p_slider->handle, p_screen_pos->x, p_screen_pos->y-4);
	setUV0(&p_slider->handle, p_texpage_pos->x, p_texpage_pos->y);

	p_slider->handle.clut = clut;

	/* initialize "fill style" slider */

#define	BOUNDARY_R	255
#define	BOUNDARY_G	255
#define	BOUNDARY_B	255

	/* FIRST setup the LINE_F2 */
	SetLineF2(&p_slider->left_boundary);
	setRGB0(&p_slider->left_boundary, BOUNDARY_R, BOUNDARY_G, BOUNDARY_B);

	/* bottom left */
	p_slider->left_boundary.x0 = p_screen_pos->x;
	p_slider->left_boundary.y0 = p_screen_pos->y + p_screen_pos->h;

	/* top left */
	p_slider->left_boundary.x1 = p_screen_pos->x;
	p_slider->left_boundary.y1 = p_screen_pos->y;

	/* THEN setup the LINE_F4 */
	SetLineF4(&p_slider->boundary);
	setRGB0(&p_slider->boundary, BOUNDARY_R, BOUNDARY_G, BOUNDARY_B);

	/* top left */
	p_slider->boundary.x0 = p_screen_pos->x;
	p_slider->boundary.y0 = p_screen_pos->y;

	/* top right */
	p_slider->boundary.x1 = p_screen_pos->x + p_screen_pos->w;
	p_slider->boundary.y1 = p_screen_pos->y;

	/* bottom right */
	p_slider->boundary.x2 = p_screen_pos->x + p_screen_pos->w;
	p_slider->boundary.y2 = p_screen_pos->y + p_screen_pos->h;

	/* bottom left */
	p_slider->boundary.x3 = p_screen_pos->x;
	p_slider->boundary.y3 = p_screen_pos->y + p_screen_pos->h;

	/* FINALLY setup the POLY_F4 */
	SetPolyF4(&p_slider->fill_in);
	setRGB0(&p_slider->fill_in, 255, 255, 255);

	/* top left */
	p_slider->fill_in.x0 = p_slider->boundary.x0+1;
	p_slider->fill_in.y0 = p_slider->boundary.y0+1;

	/* top right */
	p_slider->fill_in.x1 = p_slider->boundary.x1;
	p_slider->fill_in.y1 = p_slider->boundary.y1+1;

	/*
	 * x/y values are flipped here because POLY_F4s work like this:
	 *
	 *  (x0, y0)                (x1, y1)
	 *        ____________________
	 *       |                    |
	 *       |                    |
	 *       |                    |
	 *       |      POLY_FT4 /    |
	 *       |      POLY_F4       |
	 *       |                    |
	 *       |                    |
	 *       |____________________|
	 *
	 *  (x2, y2)                (x3, y3)
	 *
	 * while LINE_F4s directly to each point
	 */

	/* bottom left */
	p_slider->fill_in.x2 = p_slider->boundary.x3+1;
	p_slider->fill_in.y2 = p_slider->boundary.y3;

	/* bottom right */
	p_slider->fill_in.x3 = p_slider->boundary.x2;
	p_slider->fill_in.y3 = p_slider->boundary.y2;

	/* set selected style */
	p_slider->style = style;

	/* set DR_TPAGE primitive */
	SetDrawTPage(&p_slider->dr_tpage, 0, 0, tpage);
}

/*
 * slider_draw:
 *	Draws the members of a slider without an OT.
 */
void
slider_draw(p_slider)
struct Slider *p_slider;
{
	switch(p_slider->style)
	{
		case SIMPLE:
			DrawPrim(&p_slider->dr_tpage);

			p_slider->handle.x0 = p_slider->x + p_slider->current_value;

			DrawPrim(&p_slider->bar);
			DrawPrim(&p_slider->handle);
			break;
		case FILL:

			/* right side of fill_in */
			p_slider->fill_in.x1 = p_slider->x + p_slider->current_value;
			p_slider->fill_in.x3 = p_slider->x + p_slider->current_value;

			DrawPrim(&p_slider->left_boundary);
			DrawPrim(&p_slider->boundary);
			DrawPrim(&p_slider->fill_in);
			break;
	}
}

